@@ -29,7 +29,7 @@
 #include "icon-entry-hack.h"
 #include "misc.h"
 
-/* set the DOING_GDM_DEVELOPMENT env variable to "yes" if you don't
+/* set the DOING_GDM_DEVELOPMENT env variable if you don't
  * want to do that root stuff, better then something you have to change
  * in the source and recompile */
 static gboolean DOING_GDM_DEVELOPMENT = FALSE;
@@ -44,8 +44,9 @@ GladeXML *system_notebook = NULL;
 gchar *glade_filename;
 
 /* FIXME: this is a global and immutable, it should change as the
- * user changes the dir */
-char *sessions_directory;
+ * user changes the dir, but that has many problems, so for now it
+ * stays the same all the time */
+static char *sessions_directory = NULL;
 
 /* 3 user levels are present in the CList */
 gchar *basic_row[1] = { N_("Basic") };
@@ -139,12 +140,195 @@ void gdm_radio_write (gchar *radio_base_name,
    }
 }
 
+static gboolean
+gdm_event (GtkObject *object,
+	   guint signal_id,
+	   guint n_params,
+	   GtkArg *params,
+	   gpointer data)
+{
+	/* HAAAAAAAAAAAAAAAAACK */
+	/* Since the user has not logged in yet and may have left/right
+	 * mouse buttons switched, we just translate every right mouse click
+	 * to a left mouse click */
+	GdkEvent *event = GTK_VALUE_POINTER(params[0]);
+	if ((event->type == GDK_BUTTON_PRESS ||
+	     event->type == GDK_2BUTTON_PRESS ||
+	     event->type == GDK_3BUTTON_PRESS ||
+	     event->type == GDK_BUTTON_RELEASE)
+	    && event->button.button == 3)
+		event->button.button = 1;
+
+	return TRUE;
+}      
+
+static void
+check_binary (GtkEntry *entry)
+{
+	char *p;
+	char *text = g_strdup (gtk_entry_get_text (entry));
+
+	/* paranoia */
+	if (text == NULL)
+		return;
+
+	p = strchr (text, ' ');
+	if (p != NULL)
+		*p = '\0';
+
+	if ( ! gdm_string_empty (text) &&
+	    access (text, X_OK) == 0)
+		entry_set_red (GTK_WIDGET (entry), FALSE);
+	else
+		entry_set_red (GTK_WIDGET (entry), TRUE);
+
+	g_free (text);
+}
+
+static void
+check_dir (GtkEntry *entry)
+{
+	char *text = gtk_entry_get_text (entry);
+
+	/* first try access as it's a LOT faster then stat */
+	if ( ! gdm_string_empty (text) &&
+	    access (text, R_OK) == 0) {
+		struct stat sbuf;
+		/* check for this being a directory */
+		if (stat (text, &sbuf) < 0 ||
+		    ! S_ISDIR (sbuf.st_mode))
+			entry_set_red (GTK_WIDGET (entry), TRUE);
+		else
+			entry_set_red (GTK_WIDGET (entry), FALSE);
+	} else {
+		entry_set_red (GTK_WIDGET (entry), TRUE);
+	}
+}
+
+static void
+check_dirname (GtkEntry *entry)
+{
+	char *text = gtk_entry_get_text (entry);
+	char *dir;
+
+	if (text == NULL)
+		return;
+	dir = g_dirname (text);
+	if (dir == NULL)
+		return;
+
+	/* first try access as it's a LOT faster then stat */
+	if ( ! gdm_string_empty (dir) &&
+	    access (dir, R_OK) == 0) {
+		struct stat sbuf;
+		/* check for this being a directory */
+		if (stat (dir, &sbuf) < 0 ||
+		    ! S_ISDIR (sbuf.st_mode))
+			entry_set_red (GTK_WIDGET (entry), TRUE);
+		else
+			entry_set_red (GTK_WIDGET (entry), FALSE);
+	} else {
+		entry_set_red (GTK_WIDGET (entry), TRUE);
+	}
+
+	g_free (dir);
+}
+
+static void
+check_file (GtkEntry *entry)
+{
+	char *text = gtk_entry_get_text (entry);
+
+	if ( ! gdm_string_empty (text) &&
+	    access (text, R_OK) == 0) {
+		entry_set_red (GTK_WIDGET (entry), FALSE);
+	} else {
+		entry_set_red (GTK_WIDGET (entry), TRUE);
+	}
+}
+
+static void
+connect_binary_checks (void)
+{
+	int i;
+	char *binaries [] = {
+		"chooser_binary",
+		"config_binary",
+		"greeter_binary",
+		"halt_command",
+		"reboot_command",
+		"background_program",
+		"failsafe_x_server",
+		"x_keeps_crashing",
+		NULL
+	};
+	for (i = 0; binaries[i] != NULL; i++)
+		gtk_signal_connect (GTK_OBJECT (get_widget (binaries[i])),
+				    "changed",
+				    GTK_SIGNAL_FUNC (check_binary), NULL);
+}
+
+static void
+connect_dir_checks (void)
+{
+	int i;
+	char *dirs [] = {
+		"init_dir",
+		"log_dir",
+		"session_dir",
+		"pre_session_dir",
+		"post_session_dir",
+		"user_auth_dir",
+		"user_auth_fb_dir",
+		"global_faces_dir",
+		"host_images_dir",
+		NULL
+	};
+	for (i = 0; dirs[i] != NULL; i++)
+		gtk_signal_connect (GTK_OBJECT (get_widget (dirs[i])),
+				    "changed",
+				    GTK_SIGNAL_FUNC (check_dir), NULL);
+}
+
+static void
+connect_dirname_checks (void)
+{
+	int i;
+	char *dirs [] = {
+		"pid_file",
+		NULL
+	};
+	for (i = 0; dirs[i] != NULL; i++)
+		gtk_signal_connect (GTK_OBJECT (get_widget (dirs[i])),
+				    "changed",
+				    GTK_SIGNAL_FUNC (check_dirname), NULL);
+}
+
+static void
+connect_file_checks (void)
+{
+	int i;
+	char *files [] = {
+		"gnome_default_session",
+		"gtkrc_file",
+		"logo_file",
+		"background_image",
+		"default_face_file",
+		"locale_file",
+		"default_host_image_file",
+		NULL
+	};
+	for (i = 0; files[i] != NULL; i++)
+		gtk_signal_connect (GTK_OBJECT (get_widget (files[i])),
+				    "changed",
+				    GTK_SIGNAL_FUNC (check_file), NULL);
+}
+
 
 int
 main (int argc, char *argv[])
 {
-    if (g_getenv ("DOING_GDM_DEVELOPMENT") != 0 &&
-	strcasecmp_no_locale (g_getenv ("DOING_GDM_DEVELOPMENT"), "yes") == 0)
+    if (g_getenv ("DOING_GDM_DEVELOPMENT") != NULL)
 	    DOING_GDM_DEVELOPMENT = TRUE;
 
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
@@ -153,6 +337,17 @@ main (int argc, char *argv[])
     gnome_init ("gdmconfig", VERSION, argc, argv);
     glade_gnome_init();
 
+    /* If we are running under gdm parse the GDM gtkRC */
+    if (g_getenv ("RUNNING_UNDER_GDM") != NULL) {
+	    char *gtkrc;
+	    gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+	    gtkrc = gnome_config_get_string (GDM_KEY_GTKRC);
+	    gnome_config_pop_prefix ();
+	    if ( ! gdm_string_empty (gtkrc))
+		    gtk_rc_parse (gtkrc);
+	    g_free (gtkrc);
+    }
+
     /* Make sure the user is root. If not, they shouldn't be messing with 
      * GDM's configuration.
      */
@@ -174,9 +369,14 @@ main (int argc, char *argv[])
      */
 	
     if ( ! DOING_GDM_DEVELOPMENT) {
-	    glade_filename = gnome_datadir_file ("gdm/gdmconfig.glade");
-	    if (glade_filename == NULL) {	  
-		    glade_filename = g_strdup ("gdmconfig.glade");
+	    if (g_file_exists (GDM_GLADE_DIR "/gdmconfig.glade")) {
+		    glade_filename = g_strdup (GDM_GLADE_DIR
+					       "/gdmconfig.glade");
+	    } else {
+		    glade_filename = gnome_datadir_file ("gdm/gdmconfig.glade");
+		    if (glade_filename == NULL) {	  
+			    glade_filename = g_strdup ("gdmconfig.glade");
+		    }
 	    }
     } else {
 	    glade_filename = g_strdup ("gdmconfig.glade");
@@ -229,12 +429,18 @@ main (int argc, char *argv[])
 	    exit(EXIT_FAILURE);
 	}
 
+    /* connect the checker signals before parsing */
+    connect_binary_checks ();
+    connect_dir_checks ();
+    connect_dirname_checks ();
+    connect_file_checks ();
+
     /* We set most of the user interface NOT wanting signals to get triggered as
      * we do it. Then we hook up the signals, and THEN set a few remaining elements.
      * This ensures sensitivity of some widgets is correct, and that the font picker
      * gets set properly.
      */
-    gdm_config_parse_most();
+    gdm_config_parse_most(FALSE);
     glade_xml_signal_autoconnect(GUI);
 
     /* we hack up our icon entry */
@@ -252,7 +458,7 @@ main (int argc, char *argv[])
 	glade_xml_signal_autoconnect(expert_notebook);
 	glade_xml_signal_autoconnect(system_notebook);
 
-    gdm_config_parse_remaining();
+    gdm_config_parse_remaining(FALSE);
 
 	gtk_clist_column_titles_passive (GTK_CLIST (get_widget ("user_level_clist")));
    	gtk_clist_column_titles_passive (GTK_CLIST (get_widget ("server_clist")));
@@ -272,6 +478,16 @@ main (int argc, char *argv[])
     gtk_widget_set_sensitive(GTK_WIDGET(get_widget("apply_button")), FALSE);
 
     gtk_widget_show (GDMconfigurator);
+
+    /* If we are running under gdm and not in a normal session we want to
+     * treat the right mouse button like the first */
+    if (g_getenv ("RUNNING_UNDER_GDM") != NULL) {
+	    guint sid = gtk_signal_lookup ("event",
+					   GTK_TYPE_WIDGET);
+	    gtk_signal_add_emission_hook (sid,
+					  gdm_event,
+					  NULL);
+    }
     
     gtk_main ();
     return 0;
@@ -305,33 +521,54 @@ void show_about_box(void) {
 }
 
 void
-gdm_config_parse_most (void)
+gdm_config_parse_most (gboolean factory)
 {
-    void *iter; gchar *key, *value;
+    void *iter;
+    gchar *key, *value, *prefix;
     DIR *sessdir;
     struct dirent *dent;
     struct stat statbuf;
     gchar *default_session_name = NULL;
     gint linklen;
+    const char *config_file;
+
+    gtk_clist_clear (GTK_CLIST (get_widget ("sessions_clist")));
+    gtk_clist_clear (GTK_CLIST (get_widget ("server_clist")));
+    number_of_servers = 0;
+
+    if (factory)
+	    config_file = GDM_FACTORY_CONFIG_FILE;
+    else
+	    config_file = GDM_CONFIG_FILE;
 
     /* If the GDM config file does not exist, we have sensible defaults,
      * but make sure the user is warned.
      */
-    if (!g_file_exists(GDM_CONFIG_FILE))
+    if (!g_file_exists(config_file))
       {
+	      char *a_server[2];
 	      char *error = g_strdup_printf (_("The configuration file: %s\n"
 					       "does not exist! Using "
 					       "default values."),
-					     GDM_CONFIG_FILE);
+					     config_file);
 	      GtkWidget *error_dialog = gnome_error_dialog(error);
 	      g_free (error);
 	      gnome_dialog_set_parent(GNOME_DIALOG(error_dialog), 
 				      (GtkWindow *) GDMconfigurator);
 	      gnome_dialog_run_and_close(GNOME_DIALOG(error_dialog));
-      }
-    
-    gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 
+	      a_server[0] = "0";
+	      a_server[1] = "/usr/bin/X11/X";
+	      gtk_clist_append (GTK_CLIST (get_widget ("server_clist")),
+				a_server);
+	      number_of_servers = 1;
+      }
+
+    prefix = g_strdup_printf ("=%s=/", config_file);
+    gnome_config_push_prefix (prefix);
+    g_free (prefix);
+
+    g_free (sessions_directory);
     sessions_directory = gnome_config_get_string (GDM_KEY_SESSDIR);
     
     /* Fill the widgets in GDM tab */
@@ -349,6 +586,9 @@ gdm_config_parse_most (void)
     gdm_entry_set("session_dir", gnome_config_get_string (GDM_KEY_SESSDIR));
     gdm_entry_set("pre_session_dir", gnome_config_get_string (GDM_KEY_PRESESS));
     gdm_entry_set("post_session_dir", gnome_config_get_string (GDM_KEY_POSTSESS));
+    gdm_entry_set("failsafe_x_server", gnome_config_get_string (GDM_KEY_FAILSAFE_XSERVER));
+    gdm_entry_set("x_keeps_crashing", gnome_config_get_string (GDM_KEY_XKEEPSCRASHING));
+    gdm_entry_set("x_keeps_crashing_configurators", gnome_config_get_string (GDM_KEY_XKEEPSCRASHING_CONFIGURATORS));
 
     gdm_entry_set("pid_file", gnome_config_get_string (GDM_KEY_PIDFILE));
     gdm_entry_set("gnome_default_session", gnome_config_get_string (GDM_KEY_GNOMEDEFAULTSESSION));
@@ -410,6 +650,7 @@ gdm_config_parse_most (void)
     gdm_entry_set ("background_image", gnome_config_get_string (GDM_KEY_BACKGROUNDIMAGE));
     gdm_color_set ("background_color", gnome_config_get_string (GDM_KEY_BACKGROUNDCOLOR));
     gdm_toggle_set ("background_scale", gnome_config_get_bool (GDM_KEY_BACKGROUNDSCALETOFIT));
+    gdm_toggle_set ("remote_only_color", gnome_config_get_bool (GDM_KEY_BACKGROUNDREMOTEONLYCOLOR));
 
     /* Fill the widgets in Greeter tab */
     /* enable_face_browser is in parse_remaining() */
@@ -446,6 +687,7 @@ gdm_config_parse_most (void)
 	/* Ignore backups and rpmsave files */
 	if ((strstr (dent->d_name, "~")) ||
 	    (strstr (dent->d_name, ".rpmsave")) ||
+	    (strstr (dent->d_name, ".dpkg-old")) ||
 	    (strstr (dent->d_name, ".deleted")) ||
 	    (strstr (dent->d_name, ".desc")) /* description file */ ||
 	    (strstr (dent->d_name, ".orig"))) {
@@ -580,7 +822,7 @@ gdm_config_parse_most (void)
 	  {
 	      a_server[0] = key;
 	      a_server[1] = value;
-	      gtk_clist_prepend(GTK_CLIST(get_widget("server_clist")), a_server);
+	      gtk_clist_append(GTK_CLIST(get_widget("server_clist")), a_server);
 	      number_of_servers++;
 	  }
 	else
@@ -595,9 +837,19 @@ gdm_config_parse_most (void)
 
 
 void
-gdm_config_parse_remaining (void)
+gdm_config_parse_remaining (gboolean factory)
 {
-    gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+    char *prefix;
+    const char *config_file;
+
+    if (factory)
+	    config_file = GDM_FACTORY_CONFIG_FILE;
+    else
+	    config_file = GDM_CONFIG_FILE;
+
+    prefix = g_strdup_printf ("=%s=/", config_file);
+    gnome_config_push_prefix (prefix);
+    g_free (prefix);
 
     /* Ensure the XDMCP frame is the correct sensitivity */
     gdm_toggle_set("enable_xdmcp", gnome_config_get_bool(GDM_KEY_XDMCP));
@@ -613,6 +865,25 @@ gdm_config_parse_remaining (void)
 
     /* Face browser stuff */
     gdm_toggle_set("enable_face_browser", gnome_config_get_bool (GDM_KEY_BROWSER));
+
+    /* Login stuff */
+    {
+	    char *login = gnome_config_get_string (GDM_KEY_AUTOMATICLOGIN);
+	    if (gdm_string_empty (login)) {
+		    gdm_toggle_set ("enable_automatic_login", FALSE);
+	    } else {
+		    gdm_toggle_set ("enable_automatic_login", gnome_config_get_bool (GDM_KEY_AUTOMATICLOGIN_ENABLE));
+	    }
+	    g_free (login);
+
+	    login = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
+	    if (gdm_string_empty (login)) {
+		    gdm_toggle_set ("enable_timed_login", FALSE);
+	    } else {
+		    gdm_toggle_set ("enable_timed_login", gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE));
+	    }
+	    g_free (login);
+    }
     
     gnome_config_pop_prefix();
 }
@@ -682,18 +953,29 @@ run_warn_reset_dialog (void)
 
 
 
-void
-write_new_config_file                  (GtkButton *button,
-                                        gpointer         user_data)
+static gboolean
+write_config (void)
 {
     int i = -1;
     GList *tmp = NULL;
     GString *errors = NULL;
+
+    if (number_of_servers == 0 &&
+	! run_query (_("You have not defined any local servers.\n"
+		       "Usually this is not a good idea unless you\n"
+		       "are sure you do not want users to be able to\n"
+		       "log in with the graphical interface on the\n"
+		       "local console and only use the xdmcp service.\n\n"
+		       "Are you sure you wish to apply these settings?"))) {
+	    return FALSE;
+    }
    
     gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
     
     /* Write out the widget contents of the GDM tab */
+    gdm_toggle_write("enable_automatic_login", GDM_KEY_AUTOMATICLOGIN_ENABLE);
     gdm_entry_write("automatic_login", GDM_KEY_AUTOMATICLOGIN);
+    gdm_toggle_write("enable_timed_login", GDM_KEY_TIMED_LOGIN_ENABLE);
     gdm_entry_write("timed_login", GDM_KEY_TIMED_LOGIN);
     gdm_spin_write("timed_delay", GDM_KEY_TIMED_LOGIN_DELAY);
     gdm_entry_write("chooser_binary", GDM_KEY_CHOOSER);
@@ -707,6 +989,9 @@ write_new_config_file                  (GtkButton *button,
     gdm_entry_write("session_dir", GDM_KEY_SESSDIR);
     gdm_entry_write("pre_session_dir", GDM_KEY_PRESESS);
     gdm_entry_write("post_session_dir", GDM_KEY_POSTSESS);
+    gdm_entry_write("failsafe_x_server", GDM_KEY_FAILSAFE_XSERVER);
+    gdm_entry_write("x_keeps_crashing", GDM_KEY_XKEEPSCRASHING);
+    gdm_entry_write("x_keeps_crashing_configurators", GDM_KEY_XKEEPSCRASHING_CONFIGURATORS);
 
     gdm_entry_write("pid_file", GDM_KEY_PIDFILE);
     gdm_entry_write("gnome_default_session", GDM_KEY_GNOMEDEFAULTSESSION);
@@ -767,6 +1052,7 @@ write_new_config_file                  (GtkButton *button,
     gdm_color_write("background_color", GDM_KEY_BACKGROUNDCOLOR);
     gdm_entry_write("background_program", GDM_KEY_BACKGROUNDPROG);
     gdm_toggle_write("background_scale", GDM_KEY_BACKGROUNDSCALETOFIT);
+    gdm_toggle_write("remote_only_color", GDM_KEY_BACKGROUNDREMOTEONLYCOLOR);
 
     /* Write out the widget contents of the Greeter tab */
     gdm_toggle_write("enable_face_browser", GDM_KEY_BROWSER);
@@ -1025,6 +1311,8 @@ write_new_config_file                  (GtkButton *button,
     }
 
     run_warn_reset_dialog ();
+
+    return TRUE;
 }
 
 void
@@ -1033,25 +1321,57 @@ revert_settings_to_file_state (GtkMenuItem *menu_item,
 {
 	if (run_query(_("This will destroy any changes made in this session.\n"
 			"Are you sure you want to do this?"))) {
-		gdm_config_parse_most();
-		gdm_config_parse_remaining();
+		gdm_config_parse_most(FALSE);
+		gdm_config_parse_remaining(FALSE);
 	}
 }
 
+void
+revert_to_factory_settings (GtkMenuItem *menu_item,
+			    gpointer user_data)
+{
+	if (run_query (_("This will destroy any changes made in the configuration.\n"
+			"Are you sure you want to do this?"))) {
+		gdm_config_parse_most (TRUE);
+		gdm_config_parse_remaining (TRUE);
+	}
+}
+
+void
+write_new_config_file                  (GtkButton *button,
+                                        gpointer         user_data)
+{
+	write_config ();
+}
+
 void
 write_and_close (GtkButton *button,
 		 gpointer user_data)
 {
-	write_new_config_file(button, user_data);
-	exit_configurator(NULL, NULL);
+	if (write_config ())
+		exit_configurator(NULL, NULL);
 }
 
 void
 open_help_page (GtkButton *button,
 		gpointer user_data)
 {
-	gnome_warning_dialog(_("Documentation is being written but is not yet finished.\nPlease be patient."));
-	/* FIXME: ! */
+	gchar *tmp;
+	tmp = gnome_help_file_find_file ("gdmconfig", "index.html");
+	if (tmp != NULL) {
+		/* If we are running under gdm, try the help browser */
+		if (g_getenv ("RUNNING_UNDER_GDM") != NULL) {
+			char *argv[] = {
+				"gnome-help-browser",
+				tmp,
+				NULL
+			};
+			gnome_execute_async (NULL, 2, argv);
+		} else {
+			gnome_help_goto (0, tmp);
+		}
+		g_free (tmp);
+	}
 }
 
 
@@ -1084,8 +1404,8 @@ change_xdmcp_sensitivity               (GtkButton       *button,
 }
 
 void
-change_background_sensitivity_image    (GtkButton       *button,
-                                        gpointer         user_data)
+change_background_sensitivity_image (GtkButton *button,
+                                     gpointer user_data)
 {
 	g_assert (button != NULL);
 	g_assert (GTK_IS_TOGGLE_BUTTON (button));
@@ -1099,16 +1419,18 @@ change_background_sensitivity_image    (GtkButton       *button,
 }
 
 void
-change_background_sensitivity_color    (GtkButton       *button,
-                                        gpointer         user_data)
+change_background_sensitivity_none (GtkButton *button,
+				    gpointer user_data)
 {
 	g_assert (button != NULL);
 	g_assert (GTK_IS_TOGGLE_BUTTON (button));
 
 	gtk_widget_set_sensitive (get_widget ("background_color"), 
-				  GTK_TOGGLE_BUTTON (button)->active);
+				  ! GTK_TOGGLE_BUTTON (button)->active);
 	gtk_widget_set_sensitive (get_widget ("background_color_label"), 
-				  GTK_TOGGLE_BUTTON (button)->active);
+				  ! GTK_TOGGLE_BUTTON (button)->active);
+	gtk_widget_set_sensitive (get_widget ("remote_only_color"), 
+				  ! GTK_TOGGLE_BUTTON (button)->active);
 }
 
 
@@ -1120,6 +1442,35 @@ set_face_sensitivity                   (GtkButton       *button,
 			     GTK_TOGGLE_BUTTON(get_widget("enable_face_browser"))->active);
 }
 
+void
+change_automatic_sensitivity (GtkButton *button,
+			      gpointer user_data)
+{
+	g_assert (button != NULL);
+	g_assert (GTK_IS_TOGGLE_BUTTON (button));
+
+	gtk_widget_set_sensitive (get_widget ("automatic_login"), 
+				  GTK_TOGGLE_BUTTON (button)->active);
+	gtk_widget_set_sensitive (get_widget ("automatic_login_label"), 
+				  GTK_TOGGLE_BUTTON (button)->active);
+}
+
+void
+change_timed_sensitivity (GtkButton *button,
+			  gpointer user_data)
+{
+	g_assert (button != NULL);
+	g_assert (GTK_IS_TOGGLE_BUTTON (button));
+
+	gtk_widget_set_sensitive (get_widget ("timed_login"), 
+				  GTK_TOGGLE_BUTTON (button)->active);
+	gtk_widget_set_sensitive (get_widget ("timed_login_label"), 
+				  GTK_TOGGLE_BUTTON (button)->active);
+	gtk_widget_set_sensitive (get_widget ("timed_delay"), 
+				  GTK_TOGGLE_BUTTON (button)->active);
+	gtk_widget_set_sensitive (get_widget ("timed_delay_label"), 
+				  GTK_TOGGLE_BUTTON (button)->active);
+}
 
 void handle_server_add_or_edit         (gchar           *string,
 					gpointer         user_data)
@@ -1276,6 +1627,9 @@ sessions_clist_row_selected                  (GtkCList *clist,
    GdmConfigSession *sess_details = (GdmConfigSession *)
      gtk_clist_get_row_data (GTK_CLIST (get_widget ("sessions_clist")),
 			     row);
+   if (sess_details == NULL)
+	   return;
+
    /* Stop silly things happening while we fill the text widget. */
    gtk_signal_handler_block_by_func (GTK_OBJECT(get_widget("session_text")),
 				     session_text_edited, NULL);
@@ -1496,12 +1850,21 @@ modify_session_name (GtkEntry *entry, gpointer data)
 void 
 session_directory_modified (GtkEntry *entry, gpointer data) 
 {
-   /* FIXME: Ask user if they wish to reload the sessions clist based on the
-    * new directory. ISSUE: what to do if they say no, especially if the
-    * new dir is invalid. Could get icky.
-    */
-	/* Right now just run a warning */
+	char *str;
 	static gboolean shown_warning = FALSE;
+	/* FIXME: Ask user if they wish to reload the sessions clist based on
+	 * the new directory. ISSUE: what to do if they say no, especially if
+	 * the new dir is invalid. Could get icky.
+	 */
+
+	/* don't warn if we haven't really changed it */
+	str = gtk_entry_get_text (GTK_ENTRY (entry));
+	if (str != NULL &&
+	    sessions_directory != NULL &&
+	    strcmp (str, sessions_directory) == 0)
+		return;
+
+	/* Right now just run a warning */
 	if ( ! shown_warning) {
 		gnome_warning_dialog_parented
 			(_("You have modified the sessions directory.\n"
